<?php
namespace Teknisa\CRM\Factory;

use Doctrine\ORM\EntityManager;

class ParceiroNegocioFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection       = $entityManager->getConnection();
    }
    
    public function replicaInteprosp($paramsInsertInterlocutor) {
        
        $insertInterlocutor = "INSERT INTO INTEPROSP (CDPROSPECT, CDINTEPROS, IDTPINTEPROS, NMINTEPROS, DSEMAILIPROS, NRTEL1INPROS, NRORG, DSCARACINPROS, IDINPROSRECPRES)
                               VALUES(:CDPROSPECT,
                                      :CDINTEPROS,
                                      'O',
                                      :NMINTEPROS,
                                      :DSEMAILIPROS,
                                      :NRTEL1INPROS,
                                      :NRORG,
                                      :DSCARACINPROS,
                                      'S')";
        
        try{
            $std = $this->connection->prepare($insertInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function updateInteprosp($paramsInsertInterlocutor){

        $updatetInterlocutor = "UPDATE INTEPROSP SET 
                            NMINTEPROS = :NMINTEPROS, DSEMAILIPROS = :DSEMAILIPROS, NRTEL1INPROS = :NRTEL1INPROS, DSCARACINPROS = :DSCARACINPROS WHERE CDPROSPECT = :CDPROSPECT AND CDINTEPROS = :CDINTEPROS AND NRORG = :NRORG";
        
        try{
            $std = $this->connection->prepare($updatetInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getCdintepros($nrrelacionaparc, $nrorg){
        
        $selectInterlocutor = " SELECT CDINTEPROS FROM INTEPROSP WHERE NRRELACIONAPARC = '$nrrelacionaparc' and NRORG = '$nrorg' ";
        
        try{
            $std = $this->connection->prepare($selectInterlocutor);
            $std->execute();
            $cdIntepros = $std->fetchAll();
            return $cdIntepros[0]['CDINTEPROS'];
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getNewParcInteprosp($params){
        
        $select = "SELECT PN.NRPARCNEGOCIO, RP.NRRELACIONAPARC FROM
                    PARCNEGOCIO PN
                    LEFT JOIN RELACIONAPARC RP
                    ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                    AND RP.NRORG = PN.NRORG
                    LEFT JOIN INTEPROSP IP
                    ON IP.NRRELACIONAPARC = RP.NRRELACIONAPARC
                    AND IP.NRORG = RP.NRORG
                    WHERE IP.CDPROSPECT = :CDPROSPECT
                    AND IP.CDINTEPROS = :CDINTEPROS
                    AND IP.NRORG = :NRORG";
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $newparc = $std->fetchAll();
            return $newparc[0];
        }
        catch(\Exception $e){
            throw $e;
        };
    }
    
    public function deleteInteprosp($nrrelacionaparc, $nrorg){
        
        $deleteInterlocutor = "DELETE FROM INTEPROSP WHERE NRRELACIONAPARC = '$nrrelacionaparc' and NRORG = '$nrorg' ";
        
        try{
            $std = $this->connection->prepare($deleteInterlocutor);
            $std->execute();
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function atualizarCaminhoLogo(array $params) {
        $sql = "UPDATE PARCNEGOCIO
                   SET DSDIRLOGOMARCA = :DSDIRLOGOMARCA
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function checkParcHasMerge($params) {
        $sql = "
            SELECT
                (CASE
                    WHEN COUNT(1) = 0
                        THEN 'N'
                    ELSE
                        'S'
                END) HAS_MERGE
            FROM
                CRM_FUSAO_PARC
            WHERE
                NRPARCDEST = :NRPARCNEGOCIO
              AND
                NRORG = :NRORG
              AND
                IDATIVO = 'S'";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function checarParcMesmoTipo($params) {
        $sql = "
            SELECT ( CASE COUNT(N_TIPO) 
                        WHEN 1 THEN 'S' 
                        ELSE 'N' 
                    END ) MESMO_TIPO 
            FROM   (SELECT COUNT(NRPARCNEGOCIO) N_TIPO 
                    FROM   PARCNEGOCIO 
                    WHERE  NRPARCNEGOCIO IN ( :NRPARCORI, :NRPARCDEST ) 
                            AND NRORG = :NRORG 
                    GROUP  BY CDTIPOPARCPRINCIPAL)";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function findFusaoRespectiva($params) {
        $sql = "
            SELECT
                NRPARCORI,
                NRPARCDEST
            FROM
                CRM_FUSAO_PARC
            WHERE
                IDATIVO = 'S'
                    AND
                NRPARCORI = :PARC_ORI
                    AND
                NRORG = :NRORG";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function desativarFusao($params) {
        $sql = "
            UPDATE
                CRM_FUSAO_PARC
            SET
                IDATIVO         =   'N',
                CDOPERULTATU    =   :CDOPERULTATU,
                CDOPERDESFUSAO  =   :CDOPERULTATU,
                DTDESFUSAO      =   SYSDATE
            WHERE
                NRPARCORI = :NRPARCORI
                    AND
                NRPARCDEST = :NRPARCDEST
                    AND
                NRORG = :NRORG";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function findFusaoPorDestino($params) {
        $sql = "
            SELECT
                NRPARCORI,
                NRPARCDEST
            FROM
                CRM_FUSAO_PARC
            WHERE
                IDATIVO = 'S'
                    AND
                NRPARCDEST = :NRPARCDEST
                    AND
                NRORG = :NRORG";
                
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function findParcName($params) {
        $sql = "SELECT NMPRINCIPALPARC
                FROM PARCNEGOCIO
                WHERE NRPARCNEGOCIO = :NRPARC
                  AND NRORG         = :NRORG
                  AND ROWNUM        = 1";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function getOperNrParcNegocio($params) {
        $sql = "SELECT NRPARCNEGOCIO
                FROM OPERADOR
                WHERE CDOPERADOR = :CDOPERADOR
                  AND NRORG      = :NRORG
                  AND ROWNUM     = 1";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function updateInscricaoEstadual($params) {
        $sql = "UPDATE PARCNEGOCIO
                SET NRINSCESTADUAL = :NRINSCESTADUAL
                WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                  AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function checarParcEstaFundido($params) {
        $sql = "SELECT (CASE COUNT(1)
                            WHEN 0 THEN 'N'
                            ELSE 'S' END) IS_MERGED
                FROM CRM_FUSAO_PARC FUSAO
                WHERE NRPARCORI = :NRPARCNEGOCIO
                  AND FUSAO.IDATIVO = 'S'
                  AND FUSAO.NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function checkParcsAreAlreadyMerged($params) {
        $sql = "SELECT (CASE COUNT(1)
                            WHEN 0 THEN 'N'
                            ELSE 'S' END) ARE_MERGED
                FROM CRM_FUSAO_PARC FUSAO
                WHERE FUSAO.NRPARCORI  = :NRPARCORI
                  AND FUSAO.NRPARCDEST = :NRPARCDEST
                  AND FUSAO.IDATIVO    = 'S'
                  AND FUSAO.NRORG      = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    public function getCdOperador($key, $find = 'OPERADOR') {
        
        if($find == 'OPERADOR') {
            $sql = "SELECT 
                      CDOPERADOR
                    FROM
                      OPERADOR
                    WHERE
                      NRPARCNEGOCIO = :P_NRPARCNEGOCIO";
            
            $params = array(
                "P_NRPARCNEGOCIO" => $key
            );
        } else if($find == "PARCNEGOCIO") {
            $sql = "SELECT 
                      NRPARCNEGOCIO
                    FROM
                      OPERADOR
                    WHERE
                      CDOPERADOR = :P_CDOPERADOR";
            
            $params = array(
                "P_CDOPERADOR" => $key
            );
        }
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        if($find == 'OPERADOR') {
            return $std->fetchAll()[0]['CDOPERADOR'];
        } else if($find == "PARCNEGOCIO") {
            return (int)$std->fetchAll()[0]['NRPARCNEGOCIO'];
        }
    }
    
    public function updateResponsavel($row) {
        try {
            $getOperador = "SELECT CDOPERADOR, NRPARCNEGOCIO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR AND NRORG = :NRORG AND ROWNUM = 1";
            $paramsOper = array(
                "CDOPERADOR" => $row['CDOPERRESPONSAVEL'],
                "NRORG" => $row['NRORG']
            );
            
            $std1 = $this->connection->prepare($getOperador);
            $std1->execute($paramsOper);
            $operador = $std1->fetchAll();
            
            if(!empty($operador)) {
                $updateProspect = "UPDATE CRM_STATUSPARC
                                   SET NRPARCRESP = :NRPARCRESP,
                                       CDOPERRESP = :CDOPERRESP
                                   WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO 
                                         AND NRORG = :NRORG
                                         AND CRM_NRORGTRAB = :CRM_NRORGTRAB";
                
                $params = array (
                    "NRORG"             => $row['NRORG'],
                    "CRM_NRORGTRAB"     => $row['CRM_NRORGTRAB'],
                    "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO'],
                    "NRPARCRESP"        => $operador[0]['NRPARCNEGOCIO'],
                    "CDOPERRESP"        => $operador[0]['CDOPERADOR']
                );
                
                $std = $this->connection->prepare($updateProspect);
                $std->execute($params);
            }
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
    
    public function transfererirOperadores($rows) {
        try {
            $getOperador = "SELECT CDOPERADOR, NRPARCNEGOCIO FROM OPERADOR WHERE CDOPERADOR = :CDOPERADOR AND NRORG = :NRORG AND ROWNUM = 1";
            $paramsOper = array(
                "CDOPERADOR" => $rows[0]['CDOPERADORDEST'],
                "NRORG" => $rows[0]['NRORG']
            );
            
            $std1 = $this->connection->prepare($getOperador);
            $std1->execute($paramsOper);
            $operador = $std1->fetchAll();
            
            if(!empty($operador)) {
                foreach($rows as $row) {
                    $updateParc = "UPDATE 
                                     CRM_STATUSPARC
                                   SET 
                                     NRPARCRESP = :NRPARCRESP,
                                     CDOPERRESP = :CDOPERRESP
                                   WHERE 
                                     NRPARCNEGOCIO = :NRPARCNEGOCIO 
                                     AND NRORG = :NRORG
                                     AND CRM_NRORGTRAB = :CRM_NRORGTRAB";
                    
                    $params = array (
                        "NRORG"             => $row['NRORG'],
                        "CRM_NRORGTRAB"     => $row['CRM_NRORGTRAB'],
                        "NRPARCNEGOCIO"     => $row['NRPARCNEGOCIO'],
                        "NRPARCRESP"        => $operador[0]['NRPARCNEGOCIO'],
                        "CDOPERRESP"        => $operador[0]['CDOPERADOR']
                    );
                    
                    $std = $this->connection->prepare($updateParc);
                    $std->execute($params);
                }
            }
            
        }catch (\Exception $e) {   
            throw $e;
        }
    }
}