function crm00001002_Sistema_Perfil_Atividade_Controller(General, CRM_Utilities, ScreenService) {
    
    /**
     * Função para salvar um sistema ao perfil
     * de atividade em caso.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do sistema.
    */
    this.saveNewActivityProfileSystem = function(widget){
        
        var currentRow = widget.currentRow;
        
        var activityProfileSystems = currentRow.SISTEMAS;
        var rows = [];
        
        activityProfileSystems.forEach(function(system, systemIndex){
            
            rows.push(this.setActivityProfileSystemValuesInCurrentRow(widget, systemIndex));
                
        }.bind(this)); 
        
        General.callRestEngine("product#/crm_perfisistema_sistema/save", "DataSet", rows).then(function(response){
                
                ScreenService.closeSwipe().then(function(){
                    
                    widget.reload();
                    
                });
                
        }.bind(this));
    };
    
    /**
     * Função para pegar as informações do 
     * array de sistemas na currentRow, e
     * tratá-las em uma cópia da currentRow para 
     * realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro do sistema.
     * 
     * @param {int} systemIndex index do sistema
     * no array de sistemas presente na currentRow.
    */
    this.setActivityProfileSystemValuesInCurrentRow = function(widget, systemIndex){
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.__is_new = true;
        currentRow.NRSEQPERFATIV = widget.parent.parent.currentRow.NRSEQPERFATIV;
        currentRow.CDSISTEMA = currentRow.SISTEMAS[systemIndex];
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        
        return currentRow;
        
    };
    
    /**
     * Função para inserir um filtro na field
     * de sistemas e logo em seguida, atualiza
     * a field.
     * 
     * @param {object} widget widget ativa no
     * modo edit, onde está sendo mostrado o campo
     * de sistemas.
    */
    this.insertSystemsFilter = function(widget){
        
        var systemField = widget.getField("SISTEMAS");
        var systemFieldFilter = systemField.dataSourceFilter;
        
        var nrseqperfativIndex = CRM_Utilities.getFilterPosition(systemFieldFilter, "P_NRSEQPERFATIV");
        
        systemFieldFilter[nrseqperfativIndex].value = widget.parent.parent.currentRow.NRSEQPERFATIV;
        
        systemField.reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('crm00001002_Sistema_Perfil_Atividade_Controller', crm00001002_Sistema_Perfil_Atividade_Controller);
});