function crm00041000_Modalidade_Pagamento_Controller(CRM_Utilities, General, templateManager, ScreenService){
    
    /**
     * Função para habilitar a visualização 
     * do 'field group' referente ao módulo
     * selecionado.
     * 
     * @param {object} widget widget onde se
     * localiza os FieldGroups.
     * 
    */
    this.activeGroupByTheSelectedModel = function(widget){
            
        var fieldGroups = widget.fieldGroups;
        
        var currentRowNewModalityOfPayment = widget.currentRow;
        
        var indexGroupFixedPaymentDay = CRM_Utilities.getFieldGroupPosition(fieldGroups, "PAGAMENTO_FIXO_DIA");
        var groupFixedPaymentDay = fieldGroups[indexGroupFixedPaymentDay];
        
        var indexGroupFormOfPayment = CRM_Utilities.getFieldGroupPosition(fieldGroups, "FORMA_PAGAMENTO");
        var groupFormOfPayment = fieldGroups[indexGroupFormOfPayment];
        
        if(currentRowNewModalityOfPayment.IDTPFORPGTOMOD === "F"){
            
            groupFixedPaymentDay.isVisible = true;
            
            groupFormOfPayment.isVisible = false;
            this.clearValuesFieldGorup(widget, groupFormOfPayment.name);
            
        }else if(currentRowNewModalityOfPayment.IDTPFORPGTOMOD === "V"){
            
            groupFormOfPayment.isVisible = true;
            
            groupFixedPaymentDay.isVisible = false;
            this.clearValuesFieldGorup(widget, groupFixedPaymentDay.name);
            
        }else{
            
            groupFormOfPayment.isVisible = false;
            groupFixedPaymentDay.isVisible = false;
            
            this.clearValuesFieldGorup(widget, groupFormOfPayment.name);
            this.clearValuesFieldGorup(widget, groupFixedPaymentDay.name);
            
        }
        
    };
    
    /**
     * Função para ocultar os fieldGroups
     * ligados ao tipo de modalidade quando
     * não estiver em um abiente de edição
     * (para não esconder informações no modo 'view').
     * 
     * @param {object} widget widget onde se
     * localiza os fieldGroups.
    */
    this.hideFieldGroupsOfModality = function(widget){
        
        var fieldGroups = widget.fieldGroups;
        
        var indexGroupFixedPaymentDay = CRM_Utilities.getFieldGroupPosition(fieldGroups, "PAGAMENTO_FIXO_DIA");
        var groupFixedPaymentDay = fieldGroups[indexGroupFixedPaymentDay];
        
        var indexGroupFormOfPayment = CRM_Utilities.getFieldGroupPosition(fieldGroups, "FORMA_PAGAMENTO");
        var groupFormOfPayment = fieldGroups[indexGroupFormOfPayment];
        
        if(widget.currentRow.__is_new === true){
            
            groupFixedPaymentDay.isVisible = false;
            groupFormOfPayment.isVisible = false;
        }    
    }; 
    
    /**
     * Limpa os fields de um determinado FieldGroup.
     * 
     * @param {object} widget widget onde se localiza
     * o fieldGroup.
     * 
     * @param {String} fieldGroupName nome do FieldGroup
     * que terá os campos limpados.
    */
    this.clearValuesFieldGorup = function(widget, fieldGroupName){
        
        widget.fields.forEach(function(field){
            
            if(field.fieldGroup === fieldGroupName){
                CRM_Utilities.clearFieldValue(field);
            }
                
        });
    };
    
    /**
     * Função para validar valores da currentRow 
     * e salvar uma nova modalidade de pagamento,
     * mostra o field group correto no modo 'edit',
     * e bloqueia a widget 'PRAZO_PAGAMENTO' caso 
     * necessário e seta a modalidade para 'A vista'
     * caso o campo esteja vazio(BANCO MAL FEITO).
     * 
     * @param {obejct} widget widget na qual será 
     * pego os valores para inserir uma nova modalidade
     * de pagamento.
    */
    this.checkValuesAndSaveModalityPayment = function(widget){
            
        this.checkIfModeIsInSight(widget);
        
        General.widgetSave(widget).then(function(){
            
            this.hideFieldGroupsOfModality(widget);
            this.activeWidgetPaymentDeadline(widget);
            
            if(!widget.currentRow.IDTPFORPGTOMOD){
                widget.currentRow.IDTPFORPGTOMOD = "A";
            }
            
            widget.reload();
            
        }.bind(this));
    };
    
    /**
     * Função para setar 'vazio' para o campo
     * 'Tipo de modalidade' caso a modalidade selecionada
     * for 'À vista'.
     * 
     * @param {object} widget widget onde se
     * localiza a field 'Tipo de modalidade'.
    */
    this.checkIfModeIsInSight = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(currentRow.IDTPFORPGTOMOD === "A"){
            currentRow.IDTPFORPGTOMOD = "";
        }
          
    };
    
    /**
     * Função para ativar a widget 'PRAZO_PAGAMENTO'
     * quando o tipo da modalidade for 'Forma de pagamento'.
     * 
    */
    this.activeWidgetPaymentDeadline = function(){
        
        var widget = templateManager.container.getWidget('PRAZO_PAGAMENTO');
        
        var widModalityPayment = widget.parent.parent;
        
        if(widModalityPayment.currentRow.IDTPFORPGTOMOD === "V"){
            
            widget.enabled = true;
            
        }else{
            
            widget.enabled = false;
            
        }
            
    };
    
    /**
     * Função para setar os valores necessários para
     * o cadastro de um novo prazo de pagamento.
     * 
     * @param {objetc(widget)} widget widget ativa, usada
     * para o cadastro.
    */
    this.setValuesForNewPaymentDeadLine = function(widget){
    
        this.setOperatorNrorgInCurrentRow(widget);
        
        this.setModalityOfPaymentInDeadLine(widget);
        
    };
    
    /**
     * Função para setar o nrorg do operador logado
     * na currentRow da tela de cadastro de prazo de
     * pagamento.
     * 
     * @param {object(widget)} widget widget que receberá
     * o nrorg em sua currentRow.
    */
    this.setOperatorNrorgInCurrentRow = function(paymentDeadlineWidget){
        
        var currentRow = paymentDeadlineWidget.currentRow;
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.NRORG = loggedOperatorInfos.NRORG;
    };
    
    /**
     * Função para setar o código da modalidade responsável
     * pelo prazo a ser criado.
     * 
     * @param {object} widget widget 'PRAZO_PAGAMENTO'
     * 
    */
    this.setModalityOfPaymentInDeadLine = function(widget){
        
        var modPaymentCurrentRow = widget.parent.parent.currentRow;
        var deadlinePaymentCurrentRow = widget.currentRow;
        
        deadlinePaymentCurrentRow.CDMODPGTO = modPaymentCurrentRow.CDMODPGTO;
        deadlinePaymentCurrentRow.DSMODPGTO = modPaymentCurrentRow.DSMODPGTO;
        
    };
    
    /**
     * Função para inserir um filtro na widget
     * 'PRAZO_PAGAMENTO'.
     * 
     * @param {object} widModalityPayment widget princital
     * (MODALIDADE_PAGAMENTO)
    */
    this.insertDeadLineFilterValue = function(widModalityPayment){
        
        var widDeadLinePayment = templateManager.container.getWidget('PRAZO_PAGAMENTO');
        
        var CDMODPGTOfilterObjectIndex = CRM_Utilities.getFilterPosition(widDeadLinePayment.dataSourceFilter, 'CDMODPGTO');    
        
        widDeadLinePayment.dataSourceFilter[CDMODPGTOfilterObjectIndex].value = widModalityPayment.currentRow.CDMODPGTO;
    };
    
}

Configuration(function(ContextRegister){
   ContextRegister.register("crm00041000_Modalidade_Pagamento_Controller", crm00041000_Modalidade_Pagamento_Controller); 
});


