/*global Util */
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhSearch', function(templateManager, ZhKeyboardService, ShortcutDeveloperService){
	return {
		link: function($scope, $element, $attrs, $controller){
			var toSearchText = false;
			var ENTER_KEY_CODE = 13;
			var ESC_KEY_CODE = 27;
			var WIDGET_NAME = "widgetOfSearch";
			var OPTION_NEXT = "next";
			var OPTION_REFRESH = "refresh";
			var OPTION_PREV = "prev";
			var OPTION_REPLACE = "replace";
			var OPTION_REPLACE_ALL = "replaceAll";
			var SEARCH_FUNCTIONS = {
				next: getNextPosition,
				prev: getPrevPosition,
				refresh: getUpdatedPosition
			};
			var samePath = false;

			function getPositionsOfWord(value, textToSearch, rowIndex){
				var positions = [];
				textToSearch = textToSearch.replace(/\\/g, "\\\\");
				var regex = new RegExp(textToSearch, 'g');
				value.replace(regex, function(valueInArray, index){
					positions.push({
						row: rowIndex,
						column: index
					});
				});
				return positions;
			}

			function getTextPositions(textToSearch){
				var text = $scope.currentEditor.getValue();
				var items = [];
				if(text){
					if(!$scope.widget.getField('caseSensitive').value()){
						text = text.toLowerCase();
						textToSearch = textToSearch.toLowerCase();
					}
					text.split('\n').forEach(function(value,index){
						if(value.indexOf(textToSearch) >= 0){
							items = items.concat(getPositionsOfWord(value, textToSearch, index));
						}
					});
				}
				return items;
			}

			function getPrevPosition(items, row, column, text){
				column = column - text.length;
				var lastIndex = items.length - 1;
				var lastItemInText = items[lastIndex];
				var prevItem = {};
				var prevItemWasFound = items.reverse().some(function(value, index){
					prevItem = value;
					prevItem.index = lastIndex--;
					return (value.row == row && value.column < column + 1) || value.row < row;
				});
				return prevItemWasFound ? prevItem : lastItemInText;
			}

			function getNextPosition(items, row, column){
				var nextItem;
				var wasFound = !items.every(function(value, index){
					nextItem = value;
					nextItem.index = index;
					return !((value.row == row && value.column > column) || value.row > row);
				});
				return wasFound ? nextItem : items[0];
			}

			function getUpdatedPosition(items, row, column){
				var positions = { row: row, column: column, index: -1 };
				if(samePath){
					items.every(function(itemPosition, index){
						var itemWasBeforeCursor = itemPosition.row < row || (itemPosition.row == row && itemPosition.column <= column);
						if(itemWasBeforeCursor){
							positions.index = index;
						}
						return itemWasBeforeCursor;
					});
				}
				return positions;
			}

			function getPositionOfNewWord(positions, row){
				var column = $scope.currentEditor.selection.getAllRanges()[0].start.column;
				var position;
				var wasFound = !positions.every(function(value, index){
					position = value;
					position.index = index;
					return !(value.row > row || value.row == row && value.column >= column);
				});
				return wasFound ? position : positions[0];
			}

			function getTextRowAndColumn(positions, option, textToSearch, key){
				var cursorPosition = $scope.currentEditor.getCursorPosition();
				var findIn = getPositionOfNewWord;
				if(!key || key == ENTER_KEY_CODE){
					option = option || OPTION_NEXT;
					findIn = SEARCH_FUNCTIONS[option];
				}
				return findIn(positions, cursorPosition.row, cursorPosition.column - 1, textToSearch);
			}

			function selectTextInEditor(row, column, textToSearch){
				$scope.currentEditor.gotoLine(row + 1, column);
				$scope.currentEditor.selection.selectTo(row, textToSearch.length +  column);
				$scope.currentEditor.centerSelection();
			}

			function setFoundsFieldPlaceholder(index, total){
				var foundsField = $scope.widget.getField("founds");
				foundsField.placeholder = index + " of " + total;
				templateManager.updateTemplate();
			}

			function findText(textToSearch, option, key){
				var positions = getTextPositions(textToSearch);
				var positionsLength = positions.length;
				var index = 0;
				if(positionsLength > 0){
					var rowAndColumn = getTextRowAndColumn(positions, option, textToSearch, key);
					if(option != OPTION_REFRESH){
						selectTextInEditor(rowAndColumn.row, rowAndColumn.column, textToSearch);
					}
					index = rowAndColumn.index + 1;
				}
				setFoundsFieldPlaceholder(index, positionsLength);
				templateManager.updateTemplate();
			}

			function hideSearch(){
				toSearchText = false;
				templateManager.updateTemplate();
			}

			$scope.isToSearchText = function(){
				return toSearchText && !$scope.inFormMode();
			};

			$scope.refreshSearch = function(newItemPath, oldItemPath){
				samePath = newItemPath == oldItemPath;
				if($scope.isToSearchText()){
					$scope.findTextInEditor(OPTION_REFRESH);
				}
			};

			function markTextInTextEditor (option, key){
				var text = $scope.widget.getField('textToSearch').value();
				if(key == ESC_KEY_CODE){
					hideSearch();
				} else if(!text){
					setFoundsFieldPlaceholder(0,0);
				} else if(!key || key < 37 || key > 40){
					findText(text, option, key);
				}
			}

			$scope.markTextInTextEditor = markTextInTextEditor;

			$scope.findTextInEditor = Util.buildDebounceMethod(markTextInTextEditor, 200);

			function changeToSearchText(event, key){
				toSearchText = $scope.inFormMode() ? false : !toSearchText;
				templateManager.updateTemplate();
				return false;
			}
			ZhKeyboardService.bind('mod+f', changeToSearchText);
			ZhKeyboardService.bind('mod+r', changeToSearchText);
			$scope.changeToSearchText = changeToSearchText;

			$scope.$watch(
				function(){
					return $scope.isToSearchText();
				}, function(){
					$scope.DevToolsCtrl.changeHeightOfCodeEditor();
					if(toSearchText){
						$scope.DevToolsCtrl.openShortcutWidget(WIDGET_NAME);
					}
				}
			);

			function createSearchWidget(){
				var FIELDS_CONFIG = {
					textToSearch: {
						class: "05 zh-dvtools",
						template: "field/text-edit.html",
						placeholder: 'Find',
						focused: true,
						events : [{
							"name":"FieldOnKeyup",
							"code": "DevToolsController.findTextInEditor(args,'"+OPTION_NEXT+"');",
							"id": (+new Date())
						}]
					},
					searchUp: {
						class: "01 zh-dvtools",
						template: "field/button.html",
						label: "<",
						events : [{
							"name":"FieldOnClick",
							"code": "DevToolsController.findTextInEditor(args,'"+OPTION_PREV+"');",
							"id": (+new Date())
						}]
					},
					searchDown: {
						class: "01 zh-dvtools",
						template: "field/button.html",
						label: ">",
						events : [{
							"name":"FieldOnClick",
							"code": "DevToolsController.findTextInEditor(args, '"+OPTION_NEXT+"');",
							"id": (+new Date())
						}]
					},
					caseSensitive: {
						class: "01 zh-dvtools",
						template: "field/button.html",
						label: "Aa",
						events : [{
							"name":"FieldOnClick",
							"code": "DevToolsController.changeCaseSensitive();",
							"id": (+new Date())
						}]
					},
					founds: {
						class: "02 zh-dvtools",
						template: "field/text-edit.html",
						readOnly: true,
						placeholder: "0 of 0",
						events : []
					},
					textToReplace: {
						class: "05 zh-dvtools",
						template: "field/text-edit.html",
						placeholder: 'Replace with',
						focused: true,
						events : [{
							"name":"FieldOnKeyup",
							"code": "DevToolsController.findTextInEditor(args,'"+OPTION_NEXT+"');",
							"id": (+new Date())
						}]
					},
					replaceOk: {
						class: "01 zh-dvtools",
						template: "field/button.html",
						label: "✓",
						events : [{
							"name":"FieldOnClick",
							"code": "DevToolsController.replaceText();",
							"id": (+new Date())
						}]
					},
					replaceAll: {
						class: "01 zh-dvtools",
						template: "field/button.html",
						label: "all",
						events : [{
							"name":"FieldOnClick",
							"code": "DevToolsController.replaceAllText();",
							"id": (+new Date())
						}]
					}
				};
				ShortcutDeveloperService.createWidgetAndContainer(WIDGET_NAME, FIELDS_CONFIG, hideSearch);
			}
			createSearchWidget();
		}
	};
});