function DeveloperTreeService(DevToolsService){
    var self = this;
    var treeServices = {};

    this.insertTreeMode = function(mode, service){
        treeServices[mode] = service;
    };

    this.setLastItem = function(mode, item){
        treeServices[mode].setLastItem(item);
    };

    this.getLastItem = function(mode){
        return treeServices[mode].getLastItem();
    };

    this.getIndexPath = function(mode, item){
        var indexPath = item.__indexPath;
        if(!indexPath){
            var treeItems = self.getTreeItems(mode);
            indexPath = treeItems.length ? treeItems[0].__indexPath : "";
        }
        return indexPath;
    };

    this.clearLastItems = function(){
        Object.keys(treeServices).forEach(function(mode){
            treeServices[mode].setLastItem();
        });
    };

    this.setActiveItem = function(item, mode, parent){
        var cursorPosition = item.cursorPosition || { row: 0, column: 0 };
        self.setLastItem(mode, item);
        DevToolsService.setLocalActiveItem(item.item.name, item.path, mode, parent, cursorPosition);
    };

    this.setUnloadedItems = function(mode, items, setItems){
        if(setItems){
            items = treeServices[mode].getTotalUnloadedItems(items);
        }
        treeServices[mode].setUnloadedItems(items);
    };

    this.getUnloadedItems = function(mode){
        return treeServices[mode].getUnloadedItems();
    };

    this.restoreUnloadedItems = function(){
        Object.keys(treeServices).forEach(function(mode){
            if(!self.getUnloadedItems(mode)){
                var treeItem = treeServices[mode].getDefaultTreeItem();
                if(treeItem){
                    self.setUnloadedItems(mode, treeItem, true);
                }
            }
        });
    };

    this.getItemMetadata = function(item, mode){
        return treeServices[mode].getItemMetadata(item);
    };

    this.getTreeItems = function(mode){
        return treeServices[mode].getTreeItems();
    };

    this.updateTree = function(mode, items){
        treeServices[mode].updateTree(items);
    };

    this.loadEmptyTrees = function(){
        Object.keys(treeServices).forEach(function(mode){
            var treeService = treeServices[mode];
            var treeItems = treeService.getTreeItems();
            var isLoaded = treeItems && treeItems.length;
            if(!isLoaded){
                treeService.loadTree();
            }
        });
    };

    this.openTree = function(mode, pathToOpen, lastItem){
        if(treeServices[mode].openTree){
            lastItem = treeServices[mode].openTree(pathToOpen, lastItem);
        } else {
            pathToOpen.every(function(part, index){
                lastItem._opened = true;
                if(lastItem[part]){
                    lastItem = lastItem[part];
                    return true;
                }
                return false;
            });
        }
        return lastItem;
    };

    this.setTreeInWindow = function(mode, window){
        treeServices[mode].setTreeInWindow(window);
    };
}
